#include "rs232.h"
#include "timer.h"
#include "string.h"
#include "rtc.h"
#include "spi.h"
#include "interrupts.h"
#include "config.h"
#include "adc_dac.h"
#include <inarm.h>
#include <stdio.h>
//#include <string.h>

/* App-specific includes */
#include "efs.h"
#include "ls.h"

typedef signed char int8_t;


#define BIT(x) ((unsigned long)1<<x)


#define   BIT0        0x00000001
#define   BIT1        0x00000002
#define   BIT2        0x00000004
#define   BIT3        0x00000008
#define   BIT4        0x00000010
#define   BIT5        0x00000020
#define   BIT6        0x00000040
#define   BIT7        0x00000080
#define   BIT8        0x00000100
#define   BIT9        0x00000200
#define   BIT10       0x00000400
#define   BIT11       0x00000800
#define   BIT12       0x00001000
#define   BIT13       0x00002000
#define   BIT14       0x00004000
#define   BIT15       0x00008000
#define   BIT16       0x00010000
#define   BIT17       0x00020000
#define   BIT18       0x00040000
#define   BIT19       0x00080000
#define   BIT20       0x00100000
#define   BIT21       0x00200000
#define   BIT22       0x00400000
#define   BIT23       0x00800000
#define   BIT24       0x01000000
#define   BIT25       0x02000000
#define   BIT26       0x04000000
#define   BIT27       0x08000000
#define   BIT28       0x10000000
#define   BIT29       0x20000000
#define   BIT30       0x40000000
#define   BIT31       0x80000000


/* Board-specific stuff goes here */
#define   BUT1        IO0PIN_bit.P0_15
#define   BUT1_dir    IO0DIR_bit.P0_15
#define   BUT2        IO0PIN_bit.P0_16
#define   BUT2_dir    IO0DIR_bit.P0_16
#define   LED1_on     IO0CLR_bit.P0_10
#define   LED1_off    IO0SET_bit.P0_10
#define   LED1_dir    IO0DIR_bit.P0_10
#define   LED2_on     IO0CLR_bit.P0_11
#define   LED2_off    IO0SET_bit.P0_11
#define   LED2_dir    IO0DIR_bit.P0_11
#define   nCP         IO1PIN_bit.P0_24
#define   nCP_dir     IO1DIR_bit.P0_24
#define   WP          IO1PIN_bit.P0_25
#define   WP_dir      IO1DIR_bit.P0_25


#define   CCS_high    IO0SET_bit.P0_2
#define   CCS_low     IO0CLR_bit.P0_2
#define   CCS_dir     IO0DIR_bit.P0_2
#define   DCS_high    IO0SET_bit.P0_7
#define   DCS_low     IO0CLR_bit.P0_7
#define   DCS_dir     IO0DIR_bit.P0_7
#define   EEBOOT_high IO0SET_bit.P0_3
#define   EEBOOT_low  IO0CLR_bit.P0_3
#define   EEBOOT_dir  IO0DIR_bit.P0_3
#define   MP3RES_high IO0SET_bit.P0_9
#define   MP3RES_low  IO0CLR_bit.P0_9
#define   MP3RES_dir  IO0DIR_bit.P0_9

#define   DREQ        IO0PIN_bit.P0_8
#define   DREQ_dir    IO0DIR_bit.P0_8

#define FS_BUF_SIZE 513 // Do not change this. SD/MMC standart requires reads/
                        // writes in blocks of 512 bytes

EmbeddedFileSystem efs;
EmbeddedFile filer, filew;
DirList list;
unsigned short e;
int vol, oldvol;
int i,j,k;
char name[LIST_MAXLENFILENAME+1];

unsigned char buf[FS_BUF_SIZE];


// From PICDEM.NET :) //
static void UART0WriteString(char *s)
{
  char c;

  while( (c = *s++) )
    UART0WriteChar(c);
}

// XXX - change it //
void Delay (unsigned long a) {
  while (--a!=0);}

void InitVS1002(void)
{

  //Init procedure as per pg. 16 of vs10XXan.pdf
  MP3RES_low = 1;
  InitSPI(1);       // Init SPI subsystem as master
  Delay(10000);
  MP3RES_high = 1;  // Release from reset

  while (!DREQ);

  CCS_low = 1;
  TransmitChar(0x02);     // Write
  TransmitChar(0x00);     // MODE
  TransmitChar(0x08);     // Default - > SM_SDINEW on, MSB first
  TransmitChar(0x00);     // SM_SDISHARED off
  CCS_high = 1;

  Delay(100000);

  CCS_low = 1;
  TransmitChar(0x02);     // Write
  TransmitChar(0x03);     // CLOCKF
  TransmitChar(0x98);     // We are using 12.288MHz crystal->
  TransmitChar(0x00);     // Activate the internal doubler
  CCS_high = 1;

  Delay(100000);          // Some delay for the internal clock to settle

  CCS_low = 1;
  TransmitChar(0x02);     // Write
  TransmitChar(0x0B);     // VOL
  TransmitChar(0x20);     // Left
  TransmitChar(0x20);     // & Right
  CCS_high = 1;

  Delay(100000);
}

void SetVolume(unsigned char vol)
{
  while (!DREQ);
  CCS_low = 1;
  TransmitChar(0x02);
  TransmitChar(0x0B);
  TransmitChar(vol);
  TransmitChar(vol);
  CCS_high = 1;
  while (!DREQ);
}


int PlayFile( unsigned char filename[], int lenght)
{
  // XXX - nasty ugly hack here
  memcpy( name, filename, lenght-4);
  memcpy( name+lenght-4, ".MP3\0", sizeof(".MP3\0"));

  if ( file_fopen(&filer, &efs.myFs, name, 'r') == 0)
   {
      lpc2000_debug_printf("Now Playing: %s\n", name);
      while ( (( e = file_read(&filer, FS_BUF_SIZE - 1, buf)) != 0) && (BUT1) && (BUT2))
      {
        // As per pg.17 of the datasheet we should send 32 bytes
        // of data when DREQ is high. Checking it more often could
        // bring some nasty results - e.g. no sound at all
        i = 0;
        do
        {
          while (!DREQ);
          DCS_low = 1;
          for ( j=0;j<32;j++)
            TransmitChar(buf[i+j]);
          DCS_high = 1;
          i += j;
         } while (i < e);
        vol = 0xff - (GetADCValue()/4);
        if ( (abs(oldvol - vol)) > 5) // SetVol is time-consuming so don't
                                      // bother with small changes
        {
          SetVolume(vol);
          oldvol = vol;
        }
       }
        if (!BUT1) i = 1;
        else if (!BUT2) i =2;
        else i = 0;
        file_fclose( &filer );
        while (!BUT1 || !BUT2); // wait untl no button is pressed
        return i;
     }
      else
     {
        lpc2000_debug_printf("File open error");
        return -1;
     }
}


int main()
{
  //unsigned char ch1, ch2;
  int8_t res;

  FrecInit();                     //Init prescaler/divider && MAU
  InitADC();
  UART0Initialize(9600);
  UART1Initialize(9600);

  // BUT1, BUT2, CP&WP - inputs, LED1, LED2 outputs -> off;
  BUT1_dir = 0;
  BUT2_dir = 0;
  nCP_dir = 0;
  WP_dir = 0;
  LED1_off = 1;
  LED1_dir = 1;
  LED2_off = 1;
  LED2_dir = 1;

  // MP3RES, CCS, DCS, EEBOT - outputs, DREQ - input
  MP3RES_dir = 1;
  CCS_dir = 1;
  DCS_dir = 1;
  EEBOOT_dir = 1;
  DREQ_dir = 0;

  // Reset the chip and disable EEPROM boot
  CCS_high = 1;
  EEBOOT_low = 1;
  MP3RES_low = 1;


  UART0WriteString("\r\nOlimex LPC-P2148 MP3 demo\r\n");

  if (nCP) {
    UART0WriteString("Please insert a card\r\n");
    while (nCP);
    Delay(1000000);
    UART0WriteString("Card present\r\n");
  }


  InitVS1002();

  vol = 0xff - (GetADCValue()/4);
  SetVolume(vol);
  oldvol = vol;

  // Sinewave test code follows
  /*
  InitSPI(1);
  MP3RES_high = 1;
  CCS_low = 1;
  TransmitChar(0x02);
  TransmitChar(0x00);
  TransmitChar(0x08);
  TransmitChar(0x20);
  CCS_high = 1;
  while (1)
  {
    DCS_low = 1;
    TransmitChar(0x53);
    TransmitChar(0xEF);
    TransmitChar(0x6E);
    TransmitChar(0x44);
    TransmitChar(0x00);
    TransmitChar(0x00);
    TransmitChar(0x00);
    TransmitChar(0x00);
    DCS_high = 1;
    LED2_on = 1;
    Delay(1000000);
    DCS_low=1;
    TransmitChar(0x45);
    TransmitChar(0x78);
    TransmitChar(0x69);
    TransmitChar(0x74);
    TransmitChar(0x0);      // pg.15 of the AN says to send
    TransmitChar(0x0);      // 0x45 0x78 0x69 0x74 to deactivate
    TransmitChar(0x0);      // the test. pg.35 of the vs1002 manual
    TransmitChar(0x0);      // says otherwise :)
    DCS_high = 1;
    LED2_off = 1;
    Delay(1000000);
  }*/

  //LED1_on = 1;

  if ( ( res = efs_init( &efs, 0 ) ) != 0 )
  {
	lpc2000_debug_printf("failed with %i\n",res);
  }
  else
  {
	lpc2000_debug_printf("ok\n");
	ls_openDir( &list, &(efs.myFs) , "/");
        k = 1;
        i = 1;
	while ( k )
       {
                if (ls_getNext(&list) == 0)
                {
		  list.currentEntry.FileName[LIST_MAXLENFILENAME-1] = '\0';
                  do
                  {
                    i = PlayFile(list.currentEntry.FileName,sizeof(list.currentEntry.FileName));
                  } while ( i == 1);
                }
                else
                  k = 0;
        }
        fs_umount( &efs.myFs );
    }
 while(1); // infinte loop
}



